import CPUIImageData from './CPUIImageData';
import ICamera from './ICamera';
import IImageData from './IImageData';
import { IViewport } from './IViewport';
import Point2 from './Point2';
import Point3 from './Point3';
import { Scaling } from './ScalingParameters';
import StackViewportProperties from './StackViewportProperties';
import type IImage from './IImage';
import { IStackInput } from './IStackInput';
export default interface IStackViewport extends IViewport {
    modality: string;
    scaling: Scaling;
    resize: () => void;
    getFrameOfReferenceUID: () => string | undefined;
    setDefaultProperties(ViewportProperties: StackViewportProperties, imageId?: string): void;
    clearDefaultProperties(imageId?: string): void;
    setProperties({ voiRange, invert, interpolationType, rotation, colormap, }: StackViewportProperties, suppressEvents?: boolean): void;
    getDefaultProperties: (imageId?: string) => StackViewportProperties;
    getProperties: () => StackViewportProperties;
    canvasToWorld: (canvasPos: Point2) => Point3;
    worldToCanvas: (worldPos: Point3) => Point2;
    getCurrentImageIdIndex: () => number | undefined;
    getImageIds: () => string[];
    hasImageId: (imageId: string) => boolean;
    hasImageURI: (imageURI: string) => boolean;
    getCurrentImageId: () => string | undefined;
    addImages(stackInputs: Array<IStackInput>, immediateRender: boolean, suppressEvents: boolean): any;
    getImageDataMetadata(image: IImage): any;
    customRenderViewportToCanvas: () => {
        canvas: HTMLCanvasElement;
        element: HTMLDivElement;
        viewportId: string;
        renderingEngineId: string;
    };
    getImageData(): IImageData | CPUIImageData;
    getCornerstoneImage: () => IImage;
    resetToDefaultProperties(): void;
    resetProperties(): void;
    getCamera(): ICamera;
    setCamera(cameraInterface: ICamera): void;
    setStack(imageIds: Array<string>, currentImageIdIndex?: number): Promise<string>;
    resetCamera(resetPan?: boolean, resetZoom?: boolean): boolean;
    setImageIdIndex(imageIdIndex: number): Promise<string>;
    calibrateSpacing(imageId: string): void;
    getRenderer(): any;
    unsetColormap(): void;
}
